//#####################################################################
//  Copyright (c) 2011-2015 Nathan Mitchell and Eftychios Sifakis
//  This file is covered by the FreeBSD license. Please refer to the 
//  license.txt file for more information.
//#####################################################################


#ifndef __NUMBER_POLICY_H__
#define __NUMBER_POLICY_H__

template<class Tw> class Number;
template<class Twi> class Mask;


//======================================================
//
//      NUMBER POLICY: Links Number to a Mask
//
//======================================================


struct ERROR_NO_MASK_TYPE;

template<class NumberType> struct NumberPolicy{
    typedef Mask<ERROR_NO_MASK_TYPE> MASK_TYPE;
};

template<> struct NumberPolicy<Number<float> >{
    typedef Mask<bool> MASK_TYPE;
};

#if defined(ENABLE_SSE_INSTRUCTION_SET)
template<> struct NumberPolicy<Number<__m128> >{
    typedef Mask<__m128> MASK_TYPE;
};
#endif

#if defined(ENABLE_AVX_INSTRUCTION_SET)
template<> struct NumberPolicy<Number<__m256> >{
    typedef Mask<__m256> MASK_TYPE;
};
#endif

#if defined(ENABLE_MIC_INSTRUCTION_SET)
template<> struct NumberPolicy<Number<__m512> >{
    typedef Mask<__mmask16> MASK_TYPE;
};
#endif

#if defined(ENABLE_NEON_INSTRUCTION_SET)
template<> struct NumberPolicy<Number<float32x4_t> >{
    typedef Mask<float32x4_t> MASK_TYPE;
};
#endif


//======================================================
//
//      MASK POLICY: Links Mask to a common type
//
//======================================================

struct ERROR_NO_COMMON_TYPE;

template<class MaskType> struct MaskPolicy{
    typedef ERROR_NO_COMMON_TYPE MASK_EXTERNAL_TYPE;
};

template<> struct MaskPolicy<Mask<bool> >{
    typedef float MASK_EXTERNAL_TYPE;
};

#if defined(ENABLE_SSE_INSTRUCTION_SET)
template<> struct MaskPolicy<Mask<__m128> >{
    typedef float MASK_EXTERNAL_TYPE;
};
#endif

#if defined(ENABLE_AVX_INSTRUCTION_SET)
template<> struct MaskPolicy<Mask<__m256> >{
    typedef float MASK_EXTERNAL_TYPE;
};
#endif

#if defined(ENABLE_SSE_INSTRUCTION_SET)
template<> struct MaskPolicy<Mask<__m128i> >{
    typedef int MASK_EXTERNAL_TYPE;
};
#endif

#if defined(ENABLE_MIC_INSTRUCTION_SET)
template<> struct MaskPolicy<Mask<__mmask16> >{
    typedef int MASK_EXTERNAL_TYPE;
};
#endif

#if defined(ENABLE_NEON_INSTRUCTION_SET)
template<> struct MaskPolicy<Mask<float32x4_t> >{
    typedef int MASK_EXTERNAL_TYPE;
};
#endif


#endif
