//#####################################################################
//  Copyright (c) 2011-2015 Nathan Mitchell and Eftychios Sifakis
//  This file is covered by the FreeBSD license. Please refer to the 
//  license.txt file for more information.
//#####################################################################


#ifndef __KERNEL_VECTOR3_H__
#define __KERNEL_VECTOR3_H__


template<class T> struct CONST_NONREFERENCE{typedef const T TYPE;};
template<class T> struct CONST_NONREFERENCE<const T>{typedef const T TYPE;};
template<class T> struct CONST_NONREFERENCE<T&>{typedef const T TYPE;};
template<class T> struct CONST_NONREFERENCE<const T&>{typedef const T TYPE;};

template<class Tn> struct Vector3
{
    Tn x;
    Tn y;
    Tn z;

    Vector3() {}

    Vector3(const Vector3<Tn&>& v)
        :x(v.x),y(v.y),z(v.z) {}

    Vector3(const Vector3<const Tn&>& v)
        :x(v.x),y(v.y),z(v.z) {}

    explicit Vector3(const Tn& x_input,const Tn& y_input,const Tn& z_input)
        :x(x_input),y(y_input),z(z_input) {}
};

template<class Tn> struct Vector3<const Tn>
{
    const Tn x;
    const Tn y;
    const Tn z;

    explicit Vector3(const Tn& x_input,const Tn& y_input,const Tn& z_input)
        :x(x_input),y(y_input),z(z_input) {}
};

template<class Tn> struct Vector3<const Tn&>
{
    const Tn& x;
    const Tn& y;
    const Tn& z;

    explicit Vector3(const Tn& x_input,const Tn& y_input,const Tn& z_input)
        :x(x_input),y(y_input),z(z_input) {}
};

template<class Tn> struct Vector3<Tn&>
{
    Tn& x;
    Tn& y;
    Tn& z;

    explicit Vector3(Tn& x_input,Tn& y_input,Tn& z_input)
        :x(x_input),y(y_input),z(z_input) {}

    template<class Tn1>
    Vector3& operator=(const Vector3<Tn1>& v)
    {x=v.x;y=v.y;z=v.z;return *this;}
};

template<class Tn1,class Tn2> Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>
operator*(const Vector3<Tn1>& v,const Tn2& n)
{return Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>(v.x*n,v.y*n,v.z*n);}

template<class Tn1,class Tn2> Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>
operator+(const Vector3<Tn1>& v,const Tn2& n)
{return Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>(v.x+n,v.y+n,v.z+n);}

template<class Tn1,class Tn2> Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>
operator+(const Vector3<Tn1>& v1,const Vector3<Tn2>& v2)
{return Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>(v1.x+v2.x,v1.y+v2.y,v1.z+v2.z);}

template<class Tn1,class Tn2> Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>
operator/(const Vector3<Tn1>& v,const Tn2& n)
{return Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>(v.x/n,v.y/n,v.z/n);}

template<class Tn1,class Tn2> Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>
operator-(const Vector3<Tn1>& v,const Tn2& n)
{return Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>(v.x-n,v.y-n,v.z-n);}

template<class Tn1,class Tn2> Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>
operator-(const Vector3<Tn1>& v1,const Vector3<Tn2>& v2)
{return Vector3<typename CONST_NONREFERENCE<Tn1>::TYPE>(v1.x-v2.x,v1.y-v2.y,v1.z-v2.z);}

template<class Tn1,class Tn2> typename CONST_NONREFERENCE<Tn1>::TYPE 
DotProduct( const Vector3<Tn1>& v1,const Vector3<Tn2>& v2)
{return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z; }

template<class Tn1> typename CONST_NONREFERENCE<Tn1>::TYPE 
Trace(const Vector3<Tn1>& v)
{return (v.x + v.y + v.z);}

template<class Tn1> typename CONST_NONREFERENCE<Tn1>::TYPE 
Magnitude_Squared( const Vector3<Tn1>& v1)
{return v1.x*v1.x + v1.y*v1.y + v1.z*v1.z; }

#endif
