//#####################################################################
//  Copyright (c) 2011-2015 Nathan Mitchell and Eftychios Sifakis
//  This file is covered by the FreeBSD license. Please refer to the 
//  license.txt file for more information.
//#####################################################################


#ifndef SUBROUTINE_Block_Duplicate
#include "KernelCommon.h"
#else
namespace {
#endif

#include "Block_Duplicate.h"

template<class T_RAW>
#ifdef SUBROUTINE_Block_Duplicate
inline
#endif
void Block_Duplicate(const float u_compact[3][27], float u_duplicated[3][8][8])
{
    typedef float T3333[3][3][3][3];

    const T3333& u_compact_3333 = *(reinterpret_cast<const T3333*>(u_compact));

    for( int v=0; v<3; v++)
        for( int i_base=0; i_base<2; i_base++)
            for( int j_base=0; j_base<2; j_base++)
                for( int k_base=0; k_base<2; k_base++)
                    for( int i=0; i<2; i++)
                        for( int j=0; j<2; j++)
                            for( int k=0; k<2; k++)
                                {
                                    u_duplicated[v][i*4+j*2+k][i_base*4+j_base*2+k_base] =
                                        u_compact_3333[v][i+i_base][j+j_base][k+k_base];
                                }

}


#ifdef ENABLE_AVX_INSTRUCTION_SET
template<>
#ifdef SUBROUTINE_Block_Duplicate
inline
#endif
void Block_Duplicate<__m256>(const float u_compact[3][27], float u_duplicated[3][8][8])
{
    for(int v=0;v<3;v++)
    {
        __m256 rtmp;

        __m256 r1=_mm256_loadu_ps(&u_compact[v][0]);
        __m256 r2=_mm256_loadu_ps(&u_compact[v][1]);
        rtmp=_mm256_permute2f128_ps(r1,r2,0x21);
        rtmp=_mm256_permute_ps(rtmp,0x4e);
        r1=_mm256_blend_ps(r1,r2,0x0c);
        r2=_mm256_blend_ps(r2,rtmp,0x0c);
        r2=_mm256_blend_ps(r2,r1,0x30);
        r1=_mm256_blend_ps(r1,rtmp,0x30);

        __m256 r3=_mm256_loadu_ps(&u_compact[v][9]);
        __m256 r4=_mm256_loadu_ps(&u_compact[v][10]);
        rtmp=_mm256_permute2f128_ps(r3,r4,0x21);
        rtmp=_mm256_permute_ps(rtmp,0x4e);
        r3=_mm256_blend_ps(r3,r4,0x0c);
        r4=_mm256_blend_ps(r4,rtmp,0x0c);
        r4=_mm256_blend_ps(r4,r3,0x30);
        r3=_mm256_blend_ps(r3,rtmp,0x30);

        __m256 r5=_mm256_loadu_ps(&u_compact[v][18]);
        __m256 r6=_mm256_loadu_ps(&u_compact[v][19]);
        rtmp=_mm256_permute2f128_ps(r5,r6,0x21);
        rtmp=_mm256_permute_ps(rtmp,0x4e);
        r5=_mm256_blend_ps(r5,r6,0x0c);
        r6=_mm256_blend_ps(r6,rtmp,0x0c);
        r6=_mm256_blend_ps(r6,r5,0x30);
        r5=_mm256_blend_ps(r5,rtmp,0x30);

        __m256 rA=_mm256_permute2f128_ps(r1,r3,0x20);
        __m256 rB=_mm256_permute2f128_ps(r2,r4,0x20);
        __m256 rC=_mm256_permute2f128_ps(r1,r3,0x31);
        __m256 rD=_mm256_permute2f128_ps(r2,r4,0x31);

        __m256 rE=_mm256_permute2f128_ps(r3,r5,0x20);
        __m256 rF=_mm256_permute2f128_ps(r4,r6,0x20);
        __m256 rG=_mm256_permute2f128_ps(r3,r5,0x31);
        __m256 rH=_mm256_permute2f128_ps(r4,r6,0x31);

        _mm256_storeu_ps(u_duplicated[v][0],rA);
        _mm256_storeu_ps(u_duplicated[v][1],rB);
        _mm256_storeu_ps(u_duplicated[v][2],rC);
        _mm256_storeu_ps(u_duplicated[v][3],rD);
        _mm256_storeu_ps(u_duplicated[v][4],rE);
        _mm256_storeu_ps(u_duplicated[v][5],rF);
        _mm256_storeu_ps(u_duplicated[v][6],rG);
        _mm256_storeu_ps(u_duplicated[v][7],rH);
    }
}
#endif

#ifdef SUBROUTINE_Block_Duplicate
}
#else
template void Block_Duplicate<float>(const float u_compact[3][27], float u_duplicated[3][8][8]);
#ifdef ENABLE_SSE_INSTRUCTION_SET
template void Block_Duplicate<__m128>(const float u_compact[3][27], float u_duplicated[3][8][8]);
#endif
#endif
