//#####################################################################
//  Copyright (c) 2011-2015 Nathan Mitchell and Eftychios Sifakis
//  This file is covered by the FreeBSD license. Please refer to the 
//  license.txt file for more information.
//#####################################################################


#ifndef SUBROUTINE_Matrix_Transpose_Times
#include <assert.h>
#include "KernelCommon.h"
#else
namespace {
#endif

template<class Tw,class T_DATA=void,class I_DATA=void>
#ifndef SUBROUTINE_Matrix_Transpose_Times
inline
#endif
void Matrix_Transpose_Times(const T_DATA (&A)[9], const T_DATA (&B)[9], T_DATA (&C)[9])
{
    typedef enum { x11=0,x21,x31,x12,x22,x32,x13,x23,x33} Matrix_Entry;

    typedef Number<Tw> Tn;

    Tn rC11;
    Tn rC21;
    Tn rC31;
    Tn rC12;
    Tn rC22;
    Tn rC32;
    Tn rC13;
    Tn rC23;
    Tn rC33;
    Tn rA1;
    Tn rA2;
    Tn rA3;
    Tn rB;

    Vector3<Tn&> vA(rA1,rA2,rA3);
    Vector3<Tn&> vC1(rC11,rC21,rC31);
    Vector3<Tn&> vC2(rC12,rC22,rC32);
    Vector3<Tn&> vC3(rC13,rC23,rC33);

    // Use 1st column of A & B

    rA1.Load(A[x11]);
    rA2.Load(A[x12]);
    rA3.Load(A[x13]);

    rB.Load(B[x11]);
    vC1=vA*rB;

    rB.Load(B[x12]);
    vC2=vA*rB;

    rB.Load(B[x13]);
    vC3=vA*rB;

    // Use 2nd column of A & B

    rA1.Load(A[x21]);
    rA2.Load(A[x22]);
    rA3.Load(A[x23]);

    rB.Load(B[x21]);
    vC1=vC1+vA*rB;

    rB.Load(B[x22]);
    vC2=vC2+vA*rB;

    rB.Load(B[x23]);
    vC3=vC3+vA*rB;

    // Use 3rd column of A & B

    rA1.Load(A[x31]);
    rA2.Load(A[x32]);
    rA3.Load(A[x33]);

    rB.Load(B[x31]);
    vC1=vC1+vA*rB;

    rB.Load(B[x32]);
    vC2=vC2+vA*rB;

    rB.Load(B[x33]);
    vC3=vC3+vA*rB;

    // Write result

    Store(C[x11],rC11);
    Store(C[x21],rC21);
    Store(C[x31],rC31);
    
    Store(C[x12],rC12);
    Store(C[x22],rC22);
    Store(C[x32],rC32);
    
    Store(C[x13],rC13);
    Store(C[x23],rC23);
    Store(C[x33],rC33);

}

#ifndef SUBROUTINE_Matrix_Transpose_Times
#define INSTANCE_KERNEL_Matrix_Transpose_Times(WIDTH) const WIDETYPE(float,WIDTH) (&A)[9], const WIDETYPE(float,WIDTH) (&B)[9], WIDETYPE(float,WIDTH) (&C)[9]
INSTANCE_KERNEL(Matrix_Transpose_Times);
#undef INSTANCE_KERNEL_Matrix_Transpose_Times
#else
}
#endif
